/*
  Midnight, a C++ midi library to load and play midi files.
  Copyright (C) 1999 Rodrigo Reyes, reyes@chez.com

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "ExtOSSDevice.h"

ExtOSSDevice::ExtOSSDevice(char *devicename, int devnb) : OSSMidiDevice(devicename)
{
  if (devnb>=0)
    _activeDevice = devnb;
  else
    {
      _activeDevice = _lastMidiId;
    }

  for (int i=0; i<16; i++)
    {
      sendControlChange(i, CTL_MAIN_VOLUME, 127);
      sendControlChange(i, CTL_EXT_EFF_DEPTH, 0);
      sendControlChange(i, CTL_CHORUS_DEPTH, 0);
      sendControlChange(i, 0x4a, 127);
    }
  setVolume(127);
  
  SEQ_DUMPBUF();

}

void ExtOSSDevice::afterTouch(int channel, int note, int velocity)
{
  SEQ_MIDIOUT(_activeDevice, MIDI_KEY_PRESSURE + channel);
  SEQ_MIDIOUT(_activeDevice, note);
  SEQ_MIDIOUT(_activeDevice, velocity);
}

void ExtOSSDevice::noteOn(int channel, int note, int velocity)
{
  SEQ_MIDIOUT(_activeDevice, MIDI_NOTEON + channel);
  SEQ_MIDIOUT(_activeDevice, note);
  SEQ_MIDIOUT(_activeDevice, velocity);
  SEQ_DUMPBUF();
}

void ExtOSSDevice::noteOff(int channel, int note, int velocity)
{
  SEQ_MIDIOUT(_activeDevice, MIDI_NOTEOFF + channel);
  SEQ_MIDIOUT(_activeDevice, note);
  SEQ_MIDIOUT(_activeDevice, velocity);
  SEQ_DUMPBUF();
}

void ExtOSSDevice::setVolume(int volume)
{
  //  sendControlChange(0, CTL_MAIN_VOLUME, volume);
}

void ExtOSSDevice::sendBender(int channel, int data1)
{
  SEQ_MIDIOUT(_activeDevice, MIDI_PITCH_BEND + channel);
  SEQ_MIDIOUT(_activeDevice, data1 & 0x7F);
  SEQ_MIDIOUT(_activeDevice, (data1 >> 7) & 0x7F);
}

void ExtOSSDevice::sendChannelAfterTouch(int channel, int data1)
{
  SEQ_MIDIOUT(_activeDevice, MIDI_CHN_PRESSURE + channel);
  SEQ_MIDIOUT(_activeDevice, data1);

}

void ExtOSSDevice::sendPatchChange(int channel, int data1)
{
  SEQ_MIDIOUT(_activeDevice, MIDI_PGM_CHANGE + channel);
  SEQ_MIDIOUT(_activeDevice, data1);
}

void ExtOSSDevice::sendControlChange(int channel, int data1, int data2)
{
  SEQ_MIDIOUT(_activeDevice, MIDI_CTL_CHANGE + channel);
  SEQ_MIDIOUT(_activeDevice, data1);
  SEQ_MIDIOUT(_activeDevice, data2);
}

void ExtOSSDevice::terminateAll()
{
  for (int i=0; i<16; i++)
    {
      sendControlChange(i, 0x7B, 0);
      sendControlChange(i, 0x78, 0);
    }
  SEQ_DUMPBUF();
}
