/*
  Midnight, a C++ midi library to load and play midi files.
  Copyright (C) 1999 Rodrigo Reyes, reyes@chez.com

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "IOUtils.h"

//
// Dummy fonction for the configure script
// (so that it can check for a function of the library)
//
extern "C" void ioutils_dummy() { }

bool IOUtils::fillBuffer(istream &in, char *data, int length, int &curlen)
{
  curlen+=length;
  if (in.eof()==true)
	return false;
  for (int i=0; i<length; i++)
    {
      *data++ = in.get();
    }
    *data = 0;
  return true;
}
/*
int IOUtils::checkPattern(char *data, const char *pattern)
{
  int count = 0;
  while (*data == *pattern)
    {
      if (*data == 0)
	return count;

      count++;
      data++;
      pattern++;
    }
  return -1;
}
*/
int IOUtils::checkPattern(char *data, const char *pattern)
{
  int count = 0;
  while (*data == *pattern)
    {
      if (*pattern == 0)
	return count;

      if (*data == 0)
	return -1;

      count++;
      data++;
      pattern++;
    }
  if (*pattern == 0)
    return count;

  return -1;
}
bool IOUtils::checkPattern(istream &in, char *pattern, int &length)
{
  int len = strlen(pattern);
  //  length+=len;
  char buffer[len+1];
  if (fillBuffer(in, buffer, len, length))
    {
      if (checkPattern(buffer, pattern)>=0)
	return true;
    }
  return false;
}

int IOUtils::getInt32(istream &in, int &length)
{
  length+=4;
  unsigned int val = 0;

  for (int i=0; i<4; i++)
    {
      unsigned char c;
      in.get(c);
      val |= (c<<((3-i)*8));
    }
  return (int) val;
}

int IOUtils::getInt16(istream &in, int &length)
{
  length += 2;
  unsigned int val = 0;
  for (int i=0; i<2; i++)
    {
      unsigned char c;
      in.get(c);
      val |= c<<((1-i)*8);
    }
  return (int) val;
}

int IOUtils::getIntVar(istream &in, int &length)
{
  int result = 0;
  unsigned char c;

  in.get(c);
  result = c&0x7f;
  length++;
  while (c & 0x80)
    {
      length++;
      in.get(c);
      result = (result <<7) | (c & 0x7f);
    }

  return result;
}

int IOUtils::getIntVar(unsigned long value, char *buffer)
{
  char *bufptr = buffer;
  bool wr = false;
  int written = 0;

  for (int i=4; i>=0; i--)
    {
      int dep = 7*i;
      int iv = (value >> dep);
      int cv = iv & 0x7f;
      if ((cv>0) || wr || (i==0))
	{
	  wr = true;
	  if (i != 0)
	    cv |= 0x80;
	  *bufptr++ = (char) (cv & 0xFF);
	  written++;
	}
    }
  *bufptr = 0;
  return written;
}

char * IOUtils::cppstrdup(char const *str)
{
  char *nustr = new char[strlen(str) +1];
  char *nuptr = nustr;
  while( *str )
    *nuptr++ = *str++;
  *nuptr = 0;
  return nustr;
}
