/*
  Midnight, a C++ midi library to load and play midi files.
  Copyright (C) 1999 Rodrigo Reyes, reyes@chez.com

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <iostream.h>



//
// 
//

#ifndef __IOUTILS_H_
#define __IOUTILS_H_

/** The IOUtils class implements miscellaneous static methods to
    handle the i/o from/to midi files.
*/
class IOUtils
{
 public:

  /** Fills a buffer, keeping up the count of "eaten" data.
      @param in the istream flow to get the data from.
      @param data is the buffer where to insert the data.
      @param length is number of chars to wait for and put in the data buffer.
      @param curlen is a reference variable that is incremented each time a char is received.
   */
  static bool fillBuffer(istream &in, char *data, int length, int &curlength);

  /// Returns the length of "pattern" if the pattern buffer is found at the beginning of the data buffer, -1 otherwise.
  static int checkPattern(char *data, const char *pattern);

  /// Verifies that the given pattern is received by the istream.
  static bool checkPattern(istream &in, char *pattern) { int dummy; return checkPattern(in,pattern,dummy); }

  /// Verifies that the given pattern is received by the istream, and increments the length variable.
  static bool checkPattern(istream &in, char *pattern, int &length);

  /// Returns the next 32-bit value in the stream.
  static int getInt32(istream &in) { int dummy; return getInt32(in, dummy); }
  /// Returns the next 32-bit value in the stream, and increments the "length" variable.
  static int getInt32(istream &in, int &length);

  /// Returns the next 16-bit value in the stream.
  static int getInt16(istream &in) { int dummy; return getInt16(in, dummy); }
  /// Returns the next 16-bit value in the stream, and increments the "length" variable.
  static int getInt16(istream &in, int &length);

  /// Returns the next variable-length value in the stream.
  static int getIntVar(istream &in) { int dummy; return getIntVar(in, dummy); }
  /// Returns the next variable-length value in the stream, and increments the "length" variable.
  static int getIntVar(istream &in, int &length);

  /// Returns the next byte in the stream.
  static int getByte(istream &in) { int dummy; return getByte(in, dummy); }
  /// Returns the next byte in the stream, and increments the "length" variable.
  static int getByte(istream &in, int &length)
    {
      length++;
      return in.get();
    }

  /// Just like the standard c "strdup" function, but uses C++ mechanisms.
  static char *cppstrdup(char const *str);

  /** Turn a long value into a midi variable-length value. 
      The variable-length is put in the buffer parameter.
  */
  static int getIntVar(unsigned long value, char *buffer);

}///
;

#endif
