/*
  Midnight, a C++ midi library to load and play midi files.
  Copyright (C) 1999 Rodrigo Reyes, reyes@chez.com

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "MidiFile.h"

#include "MidiEvent.h"

MidiTrack::MidiTrack(istream &in) : _events()
{
  int length = 0;

  if (IOUtils::checkPattern(in, "MTrk")==false)
    {
      cerr.form("Invalid track found. Skipping the track at %x\n", in.tellg());
    }
  else length = IOUtils::getInt32(in);
  int curlength = 0;
  unsigned long offset = 0;
  int eventcount = 0;
  int runningStatus = -1;
  while (curlength < length)
    {
      MidiEvent *me = new MidiEvent(in, curlength, offset, runningStatus);

      offset += me->getDelta();
      eventcount++;
      _events.push_back(me);
      if (me->getEventCode() == me->META_END)
	{
	  curlength = length;
	}
      
#ifdef DBUG_LOADING
      cerr << "read : " << me << endl;
#endif

    }
}

MidiFile::MidiFile(istream &in) 
{
  // Check header

  if (IOUtils::checkPattern(in, "MThd")==false)
    {
      cerr << "ERROR, the MThd pattern was not found\n";
      _nbTracks = 0;
      _division = 1;
      _tracks = 0;
      _midiFileType = -1;
      return;
    }
  if (IOUtils::getInt32(in) != 6)
    return;

  _midiFileType = IOUtils::getInt16(in);
  _nbTracks = IOUtils::getInt16(in);
  _division = IOUtils::getInt16(in);
  _tracks = new MidiTrack*[_nbTracks];
  
  for (int i=0; i<_nbTracks; i++)
    {
      _tracks[i] = new MidiTrack(in);
    }
}

int MidiFile::getType()
{
  return _midiFileType;
}

