/*
  Midnight, a C++ midi library to load and play midi files.
  Copyright (C) 1999 Rodrigo Reyes, reyes@chez.com

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "MidiUtils.h"

MidiTrackEventIterator* MidiUtils::getNextTrackEvent(MidiTrackEventIterator *tracksiter[], int nbtracks)
{
  MidiTrackEventIterator *mt = 0;
  for (int i=0; i<nbtracks; i++)
    {
      MidiTrackEventIterator *curit = tracksiter[i];
      if ((mt == 0) && (curit->hasNext()))
	mt = curit;
      else 
	{
	  if (curit->hasNext())
	    {
	      if (mt->tasteNext()->getOffset() > curit->tasteNext()->getOffset())
		mt = tracksiter[i];
	    }
	}
    }
  return mt;
}


void MidiUtils::uniquifyTrack(MidiFile *file, vector<pair<unsigned long, MidiEvent*> > &seq)
{
  seq.clear();
  int tcount = file->getTrackCount();

  //
  // Alloc the track iterators 
  //
  MidiTrackEventIterator **tracksiter = new MidiTrackEventIterator*[tcount];
  for (int i=0; i<tcount; i++)
    {
      tracksiter[i] = new MidiTrackEventIterator(file, file->getTrack(i));
    }

  MidiTrackEventIterator *iter = getNextTrackEvent(tracksiter, tcount);

  while (iter != 0)
    {
      MidiEvent *me = iter->getNext();

      //      cout << "uniquified track : " << me << endl;
      /*
      if (me->getEventCode() == MidiEvent::META_TEXT_EVENT)
	{
	  int b = 34;
	  b += 34;
	}
      */
      //	  cerr << me->getOffset() << " :: " << me << "\n";

      seq.push_back( make_pair(me->getOffset(), me) );
      iter = getNextTrackEvent(tracksiter, tcount);
    }


  //
  // Clean up
  //
  for (int i=0; i<tcount; i++)
    {
      delete tracksiter[i];
    }
  delete[] tracksiter;
}

void MidiUtils::extractLyrics( vector<pair<unsigned long, MidiEvent*> > &events, vector<pair<unsigned long, MidiEvent*> > &lyrics)
{
  vector<string> foo,bar;
  string a;
  extractLyrics(events, lyrics, a, foo, bar);
}

void MidiUtils::extractLyrics( vector<pair<unsigned long, MidiEvent*> > &events, vector<pair<unsigned long, MidiEvent*> > &lyrics, string&language, vector<string>&title, vector<string>&info)
{
  vector<pair<unsigned long, MidiEvent*> >::iterator cur  = events.begin();
  vector<pair<unsigned long, MidiEvent*> >::iterator end  = events.end();

  int lyrfound = 0;
  int txtfound = 0;
  lyrics.clear();

  for (int ecode; cur != end; cur++)
    {
      ecode = cur->second->getEventCode();

      if (ecode == MidiEvent::META_LYRIC)
	{
	  char *text = ((MidiMetaTextEvent*)cur->second->getData())->getDataBuffer();
	  if ((IOUtils::checkPattern(text, "@T")>=0)
	      && (IOUtils::checkPattern(text, "@t")>=0)
	      && (IOUtils::checkPattern(text, "@L")>=0)
	      && (IOUtils::checkPattern(text, "@l")>=0)
	      && (IOUtils::checkPattern(text, "@K")>=0)
	      && (IOUtils::checkPattern(text, "@k")>=0)
	      && (IOUtils::checkPattern(text, "@V")>=0)
	      && (IOUtils::checkPattern(text, "@v")>=0)
	      && (IOUtils::checkPattern(text, "@I")>=0)
	      && (IOUtils::checkPattern(text, "@i")>=0))
	    {
	      lyrics.push_back( *cur );
	      lyrfound++;
	    }
	}
      else if (ecode == MidiEvent::META_TEXT_EVENT)
	{
	  txtfound++;
	}
    }

  if ((lyrfound >10) || (txtfound<lyrfound))
    return;
      
  lyrics.clear();
  cur  = events.begin();

  for (int ecode; cur != end; cur++)
    {
      ecode = cur->second->getEventCode();

      if (ecode==MidiEvent::META_TEXT_EVENT)
	{
	  //	  cerr << "lyrics extracted : " << cur->second << endl;
	  char *text = ((MidiMetaTextEvent*)cur->second->getData())->getDataBuffer();

	  if ((IOUtils::checkPattern(text, "@T")>=0) || (IOUtils::checkPattern(text, "@t")>=0))
	    {
	      title.push_back(string(text+2));
	    }
	  else if ((IOUtils::checkPattern(text, "@L")>=0) || (IOUtils::checkPattern(text, "@l")>=0))
	    {
	      language = text+2;
	    }
	  else if ((IOUtils::checkPattern(text, "@I")>=0) || (IOUtils::checkPattern(text, "@i")>=0))
	    {
	      info.push_back(text+2);
	    }
	  else if ((IOUtils::checkPattern(text, "@K")>=0)
		   || (IOUtils::checkPattern(text, "@k")>=0)
		   || (IOUtils::checkPattern(text, "@V")>=0)
		   || (IOUtils::checkPattern(text, "@v")>=0))
	    {
	      // just ignore these tags
	    }
	  else
	    {
	      lyrics.push_back( *cur );
	    }

	}
    }
}


char *MidiUtils::cppstrdup(char const *str)
{
  char *nustr = new char[strlen(str) +1];
  char *nuptr = nustr;
  while( *str )
    *nuptr++ = *str++;
  *nuptr = 0;
  return nustr;
}

void MidiUtils::extractLyrics(MidiFile *midif, vector<pair<unsigned long, MidiEvent*> > &lyrics, string & language, vector<string> &title, vector<string> &info)
{
  if (midif->getTrackCount() == 1)
    {
      MidiTrackEventIterator iter(midif, midif->getTrack(0));
      extractLyrics(&iter, lyrics, language, title, info, true);
      return;
    }
  else if (midif->getTrackCount()>1)
    {
      MidiTrackEventIterator iter(midif, midif->getTrack(0));
      extractLyrics(&iter, lyrics, language, title, info, false);

      MidiTrackEventIterator iter1(midif, midif->getTrack(1));
      extractLyrics(&iter1, lyrics, language, title, info, true);

      if (lyrics.size() > 1)
	return;

      int maxlyrfound = 0;
      int maxlyrfoundidx = 0;

      vector<string> dummy;
      string ddum;
      for (int i=0; i< midif->getTrackCount(); i++)
	{
	  lyrics.clear();
	  MidiTrackEventIterator iter2(midif, midif->getTrack(i));
	  extractLyrics(&iter2, lyrics, ddum, dummy, dummy, true);
	  
	  if (lyrics.size()>(unsigned int)maxlyrfound)
	    {
	      maxlyrfoundidx = i;
	      maxlyrfound = lyrics.size();
	    }
	}

	MidiTrackEventIterator iter3(midif, midif->getTrack(maxlyrfoundidx));
	lyrics.clear();
	extractLyrics(&iter3, lyrics, ddum, dummy, dummy, true);
    }
}

bool MidiUtils::extractLyrics(MidiTrackEventIterator*iter, vector<pair<unsigned long, MidiEvent*> > &lyrics, string & language, vector<string> &title, vector<string> &info, bool getlyrics)
{

  bool res = false;

  while (iter->hasNext())
    {
      MidiEvent* me = iter->getNext();
      
      if (me->getEventCode()==MidiEvent::META_TEXT_EVENT)
	{
	  //	  cerr << "lyrics extracted : " << cur->second << endl;
	  char *text = ((MidiMetaTextEvent*)me->getData())->getDataBuffer();

	  if ((IOUtils::checkPattern(text, "@T")>=0) || (IOUtils::checkPattern(text, "@t")>=0))
	    {
	      title.push_back(string(text+2));
	    }
	  else if ((IOUtils::checkPattern(text, "@L")>=0) || (IOUtils::checkPattern(text, "@l")>=0))
	    {
	      language = text+2;
	    }
	  else if ((IOUtils::checkPattern(text, "@I")>=0) || (IOUtils::checkPattern(text, "@i")>=0))
	    {
	      info.push_back(text+2);
	    }
	  else if ((IOUtils::checkPattern(text, "@K")>=0)
		   || (IOUtils::checkPattern(text, "@k")>=0)
		   || (IOUtils::checkPattern(text, "@V")>=0)
		   || (IOUtils::checkPattern(text, "@v")>=0))
	    {
	      // just ignore these tags
	    }
	  else if (getlyrics)
	    {
	      // Suppose it is the lyrics
	      res = true;
	      lyrics.push_back( make_pair(me->getOffset(), me) );
	    }

	}
    }
  return res;
}
