/*
  Midnight, a C++ midi library to load and play midi files.
  Copyright (C) 1999 Rodrigo Reyes, reyes@chez.com

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "SynthOSSDevice.h"
#include <iostream.h>

SynthOSSDevice::SynthOSSDevice(char *devicename, int devnb) : OSSMidiDevice(devicename)
{
  
  if (devnb>=0)
    _activeDevice = devnb;
  else
    {
      _activeDevice = _lastSynthId;
    }

  //
  // reset the device
  //
  ioctl(_seq, SNDCTL_SEQ_RESET);

  for (int i=0; i<16; i++)
    {
      sendControlChange(i, CTL_MAIN_VOLUME, 127);
      sendControlChange(i, CTL_EXT_EFF_DEPTH, 0);
      sendControlChange(i, CTL_CHORUS_DEPTH, 0);
      sendControlChange(i, 0x4a, 127);
  
    }
  setVolume(127);
  
  SEQ_DUMPBUF();
}

void SynthOSSDevice::afterTouch(int channel, int note, int velocity)
{
  SEQ_KEY_PRESSURE(_activeDevice, channel, note, velocity);
}

void SynthOSSDevice::noteOn(int channel, int note, int velocity)
{
  if (velocity == 0)
    {
      SEQ_STOP_NOTE(_activeDevice, channel, note, velocity);
      return;
    }
  SEQ_START_NOTE(_activeDevice, channel, note, velocity);
}

void SynthOSSDevice::noteOff(int channel, int note, int velocity)
{
  SEQ_STOP_NOTE(_activeDevice, channel, note, velocity);
}

void SynthOSSDevice::setVolume(int volume)
{
  SEQ_VOLUME_MODE(_activeDevice, volume);
}

void SynthOSSDevice::sendBender(int channel, int data1)
{
  SEQ_BENDER(_activeDevice, channel, data1);
}

void SynthOSSDevice::sendChannelAfterTouch(int channel, int data1) 
{
  SEQ_CHN_PRESSURE(_activeDevice, channel, data1);
}

void SynthOSSDevice::sendPatchChange(int channel, int data1)
{
  //  if (_mapper != NULL)
  //    data1 = _mapper->getInstrument(data1);
  SEQ_SET_PATCH(_activeDevice, channel, data1);
}

void SynthOSSDevice::sendControlChange(int channel, int data1, int data2)
{
  if ((data1==7)||(data1==39)||(data1==11))
    {
      return;
    }

  SEQ_CONTROL(_activeDevice, channel, data1, data2);
}

void SynthOSSDevice::terminateAll()
{
  //  cerr << "terminate all" << endl;
  setVolume(0);
  sync();
  ioctl(_seq, SNDCTL_SEQ_RESET);
  sync();
  setVolume(127);
}
