/*
  Midnight, a C++ midi library to load and play midi files.
  Copyright (C) 1999 Rodrigo Reyes, reyes@chez.com

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef _ALPHAKARAOKE_H_
#define _ALPHAKARAOKE_H_

#include "KaraokeBase.h"
#include "stdlib.h"
#include <gdk/gdk.h>
#include <gtk/gtk.h>

#ifdef MIDNIGHT_USE_GNOME
#include "gnome.h"
#else
#include "FontDialogPrefs.h"
#include "ColorDialogPrefs.h"
#endif

void guiplayer_data_dropped  (GtkWidget          *widget,
			      GdkDragContext     *context,
			      gint                x,
			      gint                y,
			      GtkSelectionData   *data,
			      guint               info,
			      guint               time,
			      gpointer            gplayer);


class AlphaKaraoke : public KaraokeBase
{
  static char *DEFAULTFONT;
  string _fontName;

  GtkWidget *_mainWindow;
  GtkWidget *_drawArea;

  GtkWidget* _statusBar;
  gint _statusBarContextId;

  vector<vector<pair<unsigned long, string> >::iterator> _line;
  GdkPixmap*             _backPixmap;
  GdkColor               _colorBg, _colorFgRead, _colorFgUnread, _colorFgCurRead;
  //  bool                   _colorBgAlloced, _colorFgReadAlloced, _colorFgUnreadAlloced, _colorFgCurReadAlloced;

  GdkPixmap**  _linesPixmap;
  int          _lineNumber;
  int          _lastTick;
  int          _topLineIndex;
  int          _currentLineIndex;

  GdkFont * _font;
  gint      _fontAscent;
  gint      _fontDescent;
  gint      _fontHeight;

  int       _lineHeight;

  int _areaWidth;
  int _areaHeight;

  int _pageLineUp;
  int _yoffset;
  int _yoffsetError;
  int _previousYOffset;
  int _scrollRemaining;
  int _scrollTimeSlice;
  int _scrollTimeOutID;
  timeval _scrollTimeStartedAt;
  timeval _scrollTimeOutExpiresAt;
  unsigned int _scrollTimeTotalMillis;

  int _millisBetweenScrollRefresh;

  struct LineInfo
  {
    bool skip;
    bool right;
    vector<pair<unsigned long, string> >::iterator first;
    vector<pair<unsigned long, string> >::iterator end;
    string text;
  };
  

  vector<LineInfo> _lyricsLines;

  int  _scrollType;
  int _scrollSmooth;

 public:

  enum {
    SCROLL_LINE, SCROLL_SMOOTH
  };

  enum {
    SCROLL_TOP, SCROLL_CENTER, SCROLL_FRAMED
  } ScrollType;

  AlphaKaraoke(PlayerInterface *player);

  bool _isShowing;
  virtual bool isShowing() ;

  virtual void show();
  virtual void hide();

  virtual void showPreferences();
  virtual void computeLines();
  void draw(GdkEventExpose *event);
  void updatePixmapBuffer();

  void updateLinesPixmap();

  void fillLinesPixmaps();
  void fillLinesPixmaps(int line, int pixline, bool sang);

  void fillCurrentLinePixmap(int line, int pixline, unsigned int currentTick);

  virtual void update(unsigned int ticks);
  virtual void update(){ update(_lastTick); }

  virtual void setStatus(string str);

  virtual void ticked(int tick);

  int defaultTopLineForCurrent(int curline);
  int findLineForTick(unsigned int tick);
  void redrawAllLinesPixmaps();
  void redrawCurrentLinePixmap();
  const char* strippedCharArrayFromString(string &str);
  string trimStringBegin(const string&);
  string trimString(const string&);

  string& getFontName();
  void setFont(string font, bool save);
  void ReallocColor(GdkColor*col, int r, int g, int b);
  void setReadForegroundColor(int r, int g, int b);
  GdkColor* getReadForegroundColor();
  void setUnreadForegroundColor(int r, int g, int b);
  GdkColor* getUnreadForegroundColor();
  void setCurrentForegroundColor(int r, int g, int b);
  GdkColor* getCurrentForegroundColor();
  void setBackgroundColor(int r, int g, int b);
  GdkColor* getBackgroundColor();
  
  virtual void initLyrics();

  void setScrollType(int,int);
  bool scrollIsSmooth();
  int getScrollType();

  void startScrolling(int ms, int pixels);
  int scrollArea();
  unsigned int timevalDiffMillis(timeval &first, timeval &second);

  void init()
    {
      _lastTick = 0;
      _topLineIndex = 0;
      _currentLineIndex = 0;
    }

#ifdef MIDNIGHT_USE_GTK

  class AlphaKaraokeGtkPrefs
    {
    private:
      AlphaKaraoke* _mother;
      GtkWidget* _main;
      GtkWidget* _fontSelector;
      GtkWidget* _foreUnreadColor;
      GtkWidget* _foreReadColor;
      GtkWidget* _foreCurrentColor;
      GtkWidget* _backColor;
      
      GtkWidget* _scrollSmooth;

      GtkWidget* _scrollLineFrame;
      GtkWidget* _scrollLineTop;
      GtkWidget* _scrollLineCenter;

      FontDialogPrefs* _fontDialog;
      ColorDialogPrefs** _colorDialogs;
      
    public:
      AlphaKaraokeGtkPrefs(AlphaKaraoke*mother);
      ~AlphaKaraokeGtkPrefs();
      void setFont(char *name);
      void setColor(GtkWidget*);
      void show();
      void hide();
      void apply();
      void changed();
      //      GnomePropertyBox* getPropertyBox();
      void showFontDialog();
    };

#define ALPHAKARAOKEPREFSCLASS AlphaKaraokeGtkPrefs

#endif

#ifdef MIDNIGHT_USE_GNOME

  class AlphaKaraokeGnomePrefs
    {
    private:
      AlphaKaraoke* _mother;
      GtkWidget* _main;
      GtkWidget* _fontSelector;
      GtkWidget* _foreUnreadColor;
      GtkWidget* _foreReadColor;
      GtkWidget* _foreCurrentColor;
      GtkWidget* _backColor;
      
      GtkWidget* _scrollSmooth;

      GtkWidget* _scrollLineFrame;
      GtkWidget* _scrollLineTop;
      GtkWidget* _scrollLineCenter;

    public:
      AlphaKaraokeGnomePrefs(AlphaKaraoke*mother);
      void setFont(char *name);
      void setColor(GtkWidget*);
      void show();
      void hide();
      void apply(int);
      void changed();
      GnomePropertyBox* getPropertyBox();
    };

#define ALPHAKARAOKEPREFSCLASS AlphaKaraokeGnomePrefs

#endif

};

#endif
