/*
  Midnight, a C++ midi library to load and play midi files.
  Copyright (C) 1999 Rodrigo Reyes, reyes@chez.com

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "GUIPlayer.h"
#include <iostream>
#include <string.h>
#include <stdio.h>
#include <gtk/gtkenums.h>
#include <gtk/gtkrange.h>

#include "pixmaps/play.xpm"
#include "pixmaps/pause.xpm"
#include "pixmaps/stop.xpm"
#include "pixmaps/previous.xpm"
#include "pixmaps/next.xpm"
#include "pixmaps/eject.xpm"

void guiplayer_selector_button_ok(GtkWidget *w, gpointer gui)
{
  //  cerr << "USING GUIPLAYER " << gui << endl;
  ((GUIPlayer*)gui)->selectorSelectedFile();
  /*
  */
}

void guiplayer_selector_button_cancel(GtkWidget *w, gpointer gui)
{
  ((GUIPlayer*)gui)->closeSelector();
}

void guiplayer_next_song(GtkWidget *w, gpointer gui)
{
  ((GUIPlayer*)gui)->nextSong();
}

void guiplayer_prev_song(GtkWidget *w, gpointer gui)
{
  ((GUIPlayer*)gui)->previousSong();
}

void guiplayer_play(GtkWidget *w, gpointer gui)
{
  ((GUIPlayer*)gui)->play();
}

void guiplayer_stop(GtkWidget *w, gpointer gui)
{
  ((GUIPlayer*)gui)->stop();
}

void guiplayer_pause(GtkWidget *w, gpointer gui)
{
  ((GUIPlayer*)gui)->pause();
}

void guiplayer_adjusted(GtkWidget *w, gpointer gui)
{
  if (((GUIPlayer*)gui)->_blocksetPosition == false)
    {
      ((GUIPlayer*)gui)->setPosition(GTK_ADJUSTMENT(w)->value, ".....", true);
    }
}


void guiplayer_eject(GtkWidget *widget, gpointer data)
{
  ((GUIPlayer*)data)->openSelector();

}

void guiplayer_data_dropped  (GtkWidget          *widget,
			      GdkDragContext     *context,
			      gint                x,
			      gint                y,
			      GtkSelectionData   *data,
			      guint               info,
			      guint               time,
			      gpointer            gplayer)
{
  if ((data->length >= 0) && (data->format == 8))
    {
      //      g_print ("Received \"%s\" in label\n", (gchar *)data->data);

      ((GUIPlayer*)gplayer)->droppedData((gchar *)data->data);

      gtk_drag_finish (context, TRUE, FALSE, time);
      return;
    }
  
  gtk_drag_finish (context, FALSE, FALSE, time);
}


enum {
  TARGET_STRING,
  TARGET_ROOTWIN
};

static GtkTargetEntry target_table[] = {
  { "STRING",     0, TARGET_STRING },
  { "text/plain", 0, TARGET_STRING },
  { "application/x-rootwin-drop", 0, TARGET_ROOTWIN }
};

GUIPlayer::GUIPlayer()
{
}

void GUIPlayer::init()
{
  _blocksetPosition = false;
  _filename = NULL;

  initMainWindow();
  gtk_widget_realize(_mainWindow);

  GtkStyle* style = gtk_widget_get_style(_mainWindow);
  _gdkpixPlay = gdk_pixmap_create_from_xpm_d( _mainWindow->window, &_gdkpixPlayMask, 
					      &style->bg[GTK_STATE_NORMAL], (gchar**)play_xpm);

  _gdkpixPause = gdk_pixmap_create_from_xpm_d( _mainWindow->window, &_gdkpixPauseMask, 
					       &style->bg[GTK_STATE_NORMAL], (gchar**)pause_xpm);

  gtk_window_set_policy( GTK_WINDOW(_mainWindow), TRUE, TRUE, TRUE);
  gtk_widget_set_usize( _mainWindow, 0,0);
  gtk_widget_show(_mainWindow);
  
  gtk_drag_dest_set (_mainWindow,
		     GTK_DEST_DEFAULT_ALL,
		     target_table, sizeof(target_table) / sizeof(target_table[0])-1, /* no rootwin */
		     (GdkDragAction)(GDK_ACTION_COPY | GDK_ACTION_MOVE));
  
  gtk_signal_connect( GTK_OBJECT(_mainWindow), "drag_data_received",
		      GTK_SIGNAL_FUNC( guiplayer_data_dropped ), this);

  GtkWidget *vbox = gtk_vbox_new(FALSE, 0);

  attachMenuToMain(_mainWindow, vbox);

  {
    GtkWidget *frame = gtk_frame_new( NULL);
    gtk_widget_show(frame);
    
    GtkWidget *hbox = gtk_hbox_new(TRUE, 8);

    _songInfo = gtk_label_new("");
    gtk_box_pack_start( GTK_BOX(hbox), _songInfo, FALSE, FALSE, 10);
    gtk_label_set_line_wrap( GTK_LABEL(_songInfo), TRUE);
    gtk_widget_show(_songInfo);

    gtk_widget_show(hbox);
    gtk_container_add( GTK_CONTAINER(frame), hbox );
    gtk_box_pack_start( GTK_BOX( vbox ), frame, TRUE, TRUE, 0 );

  }  

  { // Add an empty VBox so that descendant classes can add their widget inside
    _internalVBox = gtk_vbox_new(FALSE,0);
    gtk_box_pack_start( GTK_BOX(vbox), _internalVBox, FALSE, TRUE, 0);
  }

  {
    GtkWidget *hbox = gtk_hbox_new(FALSE, 10);

    _timeLabel = gtk_label_new("hh:ss");
    gtk_box_pack_start( GTK_BOX(hbox), _timeLabel, FALSE, FALSE, 6);
    gtk_widget_show(_timeLabel);

    _timeAdjustment = gtk_adjustment_new(0.0, 0.0, 100.0, 1, 10, 100);
    _timeScale = gtk_hscale_new(GTK_ADJUSTMENT(_timeAdjustment));
    gtk_scale_set_draw_value  ( GTK_SCALE(_timeScale), false );
    gtk_signal_connect( GTK_OBJECT(_timeAdjustment), "value_changed", GTK_SIGNAL_FUNC(guiplayer_adjusted), (gpointer) this);
    gtk_box_pack_start( GTK_BOX(hbox), _timeScale, TRUE, TRUE, 0);
    gtk_widget_show(_timeScale);
    //    gtk_range_set_update_policy(_timeScale, GTK_UPDATE_POLICY_CONTINUOUS);

    gtk_widget_show(hbox);
    gtk_box_pack_start( GTK_BOX( vbox ), hbox, FALSE, TRUE, 0 );
  }

  initAddCommands(_mainWindow, vbox);

  attachContentToMain(vbox);
  gtk_widget_show(vbox);

  while (gtk_events_pending())
    gtk_main_iteration();
  
  gtk_window_set_policy( GTK_WINDOW(_mainWindow), FALSE, TRUE, FALSE);


  // init others

  _fileSelector = NULL;

}


/*
void GUIPlayer::show()
{
  gtk_widget_show(_mainWindow);
}
*/
void GUIPlayer::openSelector()
{
  if (_fileSelector == NULL)
    {
      _fileSelector = gtk_file_selection_new("Choose a file");
      gtk_signal_connect( GTK_OBJECT(GTK_FILE_SELECTION(_fileSelector)->ok_button),
			  "clicked", GTK_SIGNAL_FUNC(guiplayer_selector_button_ok), this);
      gtk_signal_connect( GTK_OBJECT(GTK_FILE_SELECTION(_fileSelector)->cancel_button),
			  "clicked", GTK_SIGNAL_FUNC(guiplayer_selector_button_cancel), this);
      gtk_signal_connect( GTK_OBJECT(_fileSelector),
			  "destroy", GTK_SIGNAL_FUNC(guiplayer_selector_button_cancel), this);

      //      cerr << "SET GUI PLAYER = " << this << endl;

      if (getFileName() == NULL)
	gtk_file_selection_complete(GTK_FILE_SELECTION(_fileSelector), "*.mid");
      else gtk_file_selection_complete(GTK_FILE_SELECTION(_fileSelector), getFileName());
      gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(_fileSelector));


      
      gtk_widget_show(_fileSelector);  
    }
  else
    {
      gtk_widget_show(_fileSelector);  
    }
}

GtkFileSelection *GUIPlayer::getSelector()
{
  return GTK_FILE_SELECTION( _fileSelector );
}
void GUIPlayer::closeSelector()
{
  if (_fileSelector != NULL)
    {
      GtkWidget  *temp = _fileSelector;
      _fileSelector = NULL;
      gtk_widget_destroy(temp);
    }
}

void GUIPlayer::setFileName(const char* name)
{
  delete _filename;
  _filename = new char[ strlen(name) +1 ];
  strcpy(_filename, name);
}

void GUIPlayer::selectorSelectedFile()
{
  const char *sel = gtk_file_selection_get_filename(GTK_FILE_SELECTION(_fileSelector));
  setFileName(sel);
  closeSelector();
  newFileSelected();
}

void GUIPlayer::setSongInfo(const char *info)
{
  gtk_label_set_text( GTK_LABEL(_songInfo), info );
}

void GUIPlayer::newFileSelected() { }
void GUIPlayer::previousSong() { }
void GUIPlayer::nextSong() { }
void GUIPlayer::play() { }
void GUIPlayer::pause() { }
void GUIPlayer::stop() 
{ 
  setPosition(0,"00:00", false);
}

const char* GUIPlayer::getFileName() { return _filename; }

void GUIPlayer::adjustTimeScale(unsigned long maxTicks)
{
  GTK_ADJUSTMENT(_timeAdjustment)->lower = 0;
  GTK_ADJUSTMENT(_timeAdjustment)->upper = maxTicks;
  GTK_ADJUSTMENT(_timeAdjustment)->page_size = 1 ;
  gtk_adjustment_changed( GTK_ADJUSTMENT(_timeAdjustment));
  //      gtk_adjustment_clamp_page(GTK_ADJUSTMENT(_timeAdjustment), 0, maxTicks);
}

void GUIPlayer::setPosition(unsigned long ticks, const char *timestr, bool usermodif)
{
  if (_blocksetPosition == true)
    {
      return;
    }
  _blocksetPosition = true;

  gtk_label_set_text( GTK_LABEL(_timeLabel), timestr );
      
  if (!usermodif)
    {
      gtk_adjustment_set_value( GTK_ADJUSTMENT(_timeAdjustment), ticks );
    }
  else
    {
      setSongAt(ticks);
    }

  _blocksetPosition = false;

}

GtkWidget *GUIPlayer::getWidget() { return _mainWindow; }
void GUIPlayer::initMainWindow()
{
  _mainWindow = gtk_window_new( GTK_WINDOW_TOPLEVEL );
}

/**
   Must set the _playPixmap widget to a valid pixmap. This is supposed
   to be changed when the user click on the play button while a song
   is playing, so that another pixmap (pause) can be displayed
*/
void GUIPlayer::initAddCommands(GtkWidget* main, GtkWidget* vbox)
{
  GtkWidget *hbox = gtk_hbox_new(TRUE, 0);
  GtkWidget *button;

  GtkStyle *style;
  GdkPixmap *pixmap;
  GdkBitmap *mask;
  GtkWidget *wdpix;


  button = gtk_button_new();
  gtk_box_pack_start( GTK_BOX( hbox ), button, TRUE, TRUE, 0 );
  gtk_signal_connect( GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(guiplayer_prev_song), (gpointer) this);
  gtk_widget_show(button);


  style = gtk_widget_get_style(_mainWindow);
  pixmap = gdk_pixmap_create_from_xpm_d( _mainWindow->window, &mask, 
					 &style->bg[GTK_STATE_NORMAL], (gchar**)previous_xpm);
  wdpix = gtk_pixmap_new(pixmap, mask);
  gtk_widget_show(wdpix);
  gtk_container_add( GTK_CONTAINER(button), wdpix);


  button = gtk_button_new();
  gtk_box_pack_start( GTK_BOX( hbox ), button, TRUE, TRUE, 0 );
  gtk_signal_connect( GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(guiplayer_stop), (gpointer) this);
  gtk_widget_show(button);

  style = gtk_widget_get_style(_mainWindow);
  pixmap = gdk_pixmap_create_from_xpm_d( _mainWindow->window, &mask, 
					 &style->bg[GTK_STATE_NORMAL], (gchar**)stop_xpm);
  wdpix = gtk_pixmap_new(pixmap, mask);
  gtk_widget_show(wdpix);
  gtk_container_add( GTK_CONTAINER(button), wdpix);

  button = gtk_button_new();
  gtk_box_pack_start( GTK_BOX( hbox ), button, TRUE, TRUE, 0 );
  gtk_signal_connect( GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(guiplayer_play), (gpointer) this);
  gtk_widget_show(button);

  _playPixmap = gtk_pixmap_new(_gdkpixPlay, _gdkpixPlayMask);
  gtk_widget_show(_playPixmap);
  gtk_container_add( GTK_CONTAINER(button), _playPixmap);

  button = gtk_button_new();
  gtk_box_pack_start( GTK_BOX( hbox ), button, TRUE, TRUE, 0 );
  gtk_signal_connect( GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(guiplayer_next_song), (gpointer) this);
  gtk_widget_show(button);
  style = gtk_widget_get_style(_mainWindow);
  pixmap = gdk_pixmap_create_from_xpm_d( _mainWindow->window, &mask, 
					 &style->bg[GTK_STATE_NORMAL], (gchar**)next_xpm);
  wdpix = gtk_pixmap_new(pixmap, mask);
  gtk_widget_show(wdpix);
  gtk_container_add( GTK_CONTAINER(button), wdpix);

  button = gtk_button_new();
  gtk_box_pack_start( GTK_BOX( hbox ), button, TRUE, TRUE, 0 );
  gtk_signal_connect( GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(guiplayer_eject), (gpointer) this);
  gtk_widget_show(button);

  style = gtk_widget_get_style(_mainWindow);
  pixmap = gdk_pixmap_create_from_xpm_d( _mainWindow->window, &mask, 
					 &style->bg[GTK_STATE_NORMAL], (gchar**)eject_xpm);
  wdpix = gtk_pixmap_new(pixmap, mask);
  gtk_widget_show(wdpix);
  gtk_container_add( GTK_CONTAINER(button), wdpix);


  gtk_widget_show(hbox);
  gtk_box_pack_start( GTK_BOX( vbox ), hbox, FALSE, TRUE, 0 );
}

void GUIPlayer::attachContentToMain(GtkWidget* vbox)
{
  gtk_container_add( GTK_CONTAINER(_mainWindow), vbox);
}

void GUIPlayer::attachMenuToMain(GtkWidget* main, GtkWidget* vbox)
{

}
