/*
  Midnight, a C++ midi library to load and play midi files.
  Copyright (C) 1999 Rodrigo Reyes, reyes@chez.com

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


#ifndef _GUIPLAYER_H_
#define _GUIPLAYER_H_

#include <gtk/gtk.h>
#include <map>
#include <string>
#include "MidiDevice.h"
#include "MidiFile.h"
#include "MidiPlayer.h"
#include "StringTokenizer.h"
#include "PlayerInterface.h"

void guiplayer_next_song(GtkWidget *w, gpointer gui);
void guiplayer_prev_song(GtkWidget *w, gpointer gui);
void guiplayer_play(GtkWidget *w, gpointer gui);
void guiplayer_stop(GtkWidget *w, gpointer gui);
void guiplayer_pause(GtkWidget *w, gpointer gui);
void guiplayer_eject(GtkWidget *widget, gpointer data);

class GUIPlayer : public PlayerInterface
{
  friend  void guiplayer_selector_button_ok(GtkWidget *w, gpointer gui);
  friend  void guiplayer_selector_button_cancel(GtkWidget *w, gpointer gui);

  friend void guiplayer_data_dropped  (GtkWidget          *widget,
				       GdkDragContext     *context,
				       gint                x,
				       gint                y,
				       GtkSelectionData   *data,
				       guint               info,
				       guint               time,
				       gpointer            dplay);

  GtkWidget *_fileSelector;

  char*  _filename;

  GtkObject* _timeAdjustment;
  GtkWidget* _timeLabel;
  GtkWidget* _timeScale;
  GtkWidget* _songInfo;

 protected:
  GtkWidget* _playPixmap;

  GdkPixmap* _gdkpixPlay;
  GdkBitmap* _gdkpixPlayMask;
  GdkPixmap* _gdkpixPause;
  GdkBitmap* _gdkpixPauseMask;

 protected:

  GtkWidget* _mainWindow;

  GtkWidget* _internalMenuBarLocation;
  GtkWidget* _internalVBox;

  virtual void setPlayPixAsPlay() { gtk_pixmap_set( GTK_PIXMAP(_playPixmap), _gdkpixPlay, _gdkpixPlayMask); }
  virtual void setPlayPixAsPause() { gtk_pixmap_set( GTK_PIXMAP(_playPixmap), _gdkpixPause, _gdkpixPauseMask); }
 
  virtual void initMainWindow();

 public:
  GUIPlayer();
  virtual ~GUIPlayer() 
    {
    }
  //  void show();

  virtual GtkWidget *getWidget();
  

  
  void openSelector();
  
  virtual void selectSong(string& filename) { };
  virtual void newFileSelected();
  virtual void previousSong();
  virtual void nextSong() ;
  virtual void play() ;
  virtual void pause() ;
  virtual void stop() ;

  
  const char* getFileName();
  void setFileName(const char*name);
  void setSongInfo(const char *info);

  void adjustTimeScale(unsigned long maxTicks);

  bool _blocksetPosition;
  void setPosition(unsigned long ticks, const char *timestr, bool usermodif);

  virtual void setSongAt(unsigned long ticks) { }

  virtual void addToList(vector<string> &lst) 
    {
      //      for (vector<string>::iterator i=lst.begin(); i!= lst.end(); i++)
      //	{
      //	  cerr << "FILES : " << *i << endl;
      //	}
    }

 private:
  GtkFileSelection *getSelector();
  void closeSelector();
  void selectorSelectedFile();

  void droppedData(const char *data)
    {
      StringTokenizer st(data,"\n\r\t");
      vector<string> lst;

      while (st.hasNext())
	{
	  string cur = st.next();
	  
	  if (cur.find("file://")==0)
	    {
	      lst.push_back(cur.substr(7, cur.size()-7));
	    }
	  else if (cur.find("file:")==0)
	    {
	      lst.push_back(cur.substr(5, cur.size()-5));
	    }
	  else 
	    {
	      lst.push_back(cur);
	    }
	  
	}

      addToList(lst);
    }

 protected:
  virtual void initAddCommands(GtkWidget* main, GtkWidget* vbox);
  virtual void attachContentToMain(GtkWidget* vbox);
  virtual void attachMenuToMain(GtkWidget* main, GtkWidget* vbox);
 public:
  virtual void init();


  virtual void setFeature(string, string) = 0;
  virtual string getFeature(string, string) = 0;
  virtual void setIntFeature(string, int) = 0;
  virtual int getIntFeature(string, int) = 0;

};

#endif
