/*
  Midnight, a C++ midi library to load and play midi files.
  Copyright (C) 1999 Rodrigo Reyes, reyes@chez.com

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "GnomeGUIPlayer.h"

#include "pixmaps/play.xpm"
#include "pixmaps/pause.xpm"
#include "pixmaps/stop.xpm"
#include "pixmaps/previous.xpm"
#include "pixmaps/next.xpm"
#include "pixmaps/eject.xpm"

void mguip_toolbar_has_text_label(GtkWidget *widget, gpointer data)
{
  gnome_preferences_set_toolbar_labels(GTK_CHECK_MENU_ITEM(widget)->active);
  gnome_preferences_save();
  ((GnomeGUIPlayer*)data)->setToolbarHasText(GTK_CHECK_MENU_ITEM(widget)->active);
}

void mguip_remember_layout_configuration(GtkWidget *widget, gpointer data)
{
  ((GnomeGUIPlayer*)data)->setRememberLayout(GTK_CHECK_MENU_ITEM(widget)->active);
}

void gguip_exit(GtkWidget *widget, gpointer data)
{
  gtk_main_quit();
}

void gguip_about(GtkWidget* widget, gpointer data)
{
  GtkWidget* about;
  const char *authors[] = { "Rodrigo Reyes", NULL };
  about = gnome_about_new("GMidnight", PROGRAM_VERSION, "Copyright 1999 Rodrigo Reyes", authors,
			  "This program is released under the Gnu General Public Licence, and comes with"
			  "ABSOLUTELY NO WARRANTY; "
			  "This is free software, and you are welcome to redistribute it"
			  " under certain conditions. See http://www.gnu.org for more details on the Gnu GPL licence.",
			  NULL);
  gtk_widget_show(about);
}

GnomeGUIPlayer::GnomeGUIPlayer(MidnightPlayerManager* manager) : MidnightGUIPlayer(manager)
{
}
GnomeGUIPlayer::~GnomeGUIPlayer()
{
}

void GnomeGUIPlayer::initAddCommands(GtkWidget* main, GtkWidget* vbox)
{
  GtkWidget* playtoolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
  gtk_toolbar_append_item(GTK_TOOLBAR(playtoolbar), "Previous", "Go to the previous song in the list", "", gnome_pixmap_new_from_xpm_d( previous_xpm ), GTK_SIGNAL_FUNC(guiplayer_prev_song), this);
  gtk_toolbar_append_item(GTK_TOOLBAR(playtoolbar), "Stop", "Stop the song", "", gnome_pixmap_new_from_xpm_d( stop_xpm ), GTK_SIGNAL_FUNC(guiplayer_stop), this);
  _playPixmap = gnome_pixmap_new_from_xpm_d( play_xpm );
  gtk_toolbar_append_item(GTK_TOOLBAR(playtoolbar), "Play", "Play the song", "", _playPixmap, GTK_SIGNAL_FUNC(guiplayer_play), this);
  gtk_toolbar_append_item(GTK_TOOLBAR(playtoolbar), "Next", "Go to the next song in the list", "", gnome_pixmap_new_from_xpm_d( next_xpm ), GTK_SIGNAL_FUNC(guiplayer_next_song), this);
  gtk_toolbar_append_item(GTK_TOOLBAR(playtoolbar), "Select", "Select a new song", "", gnome_pixmap_new_from_xpm_d( eject_xpm ), GTK_SIGNAL_FUNC(guiplayer_eject), this);

  _commandToolbar = playtoolbar;
  gnome_app_set_toolbar(GNOME_APP(main), GTK_TOOLBAR(playtoolbar));

}

void GnomeGUIPlayer::init()
{
  MidnightGUIPlayer::init();

  // post-initialisations
  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(_tbHasTextCheckMenuItem), gnome_preferences_get_toolbar_labels());
}

void GnomeGUIPlayer::initMainWindow()
{
  _mainWindow = gnome_app_new("GMidnight", "GMidnight");
}

void GnomeGUIPlayer::attachContentToMain(GtkWidget* vbox)
{
  gnome_app_set_contents( GNOME_APP(_mainWindow), vbox);
}

void GnomeGUIPlayer::attachMenuToMain(GtkWidget* main, GtkWidget* vbox)
{
  GnomeUIInfo filemenus[] = { 
    GNOMEUIINFO_MENU_OPEN_ITEM(mguip_open,this),
    GNOMEUIINFO_TOGGLEITEM_DATA("Show list panel", "Show the panel that displays the list of song", mguip_show_hide_list, this, NULL),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_MENU_EXIT_ITEM(gguip_exit, this),
    GNOMEUIINFO_END };

  GnomeUIInfo karamenus[] = {
    GNOMEUIINFO_TOGGLEITEM_DATA("Show karaoke panel", "Show the Karaoke panel", mguip_show_hide_panel, this, NULL),
    GNOMEUIINFO_ITEM_DATA("Karaoke prefs...", "Set the preferences for the karaoke panel", mguip_karaoke_prefs, this, NULL),
        GNOMEUIINFO_END };

  GnomeUIInfo midiconfigmenus[] = {
    GNOMEUIINFO_ITEM_DATA("Select device...", "Select the midi device", mguip_select_device, this, NULL),
    GNOMEUIINFO_ITEM_DATA("Save Midi configuration", "Save the current midi device configuration", mguip_save_config, this, NULL),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_TOGGLEITEM_DATA("Toolbar has text label", NULL, mguip_toolbar_has_text_label, this, NULL),
    GNOMEUIINFO_TOGGLEITEM_DATA("Remember layout configuration", NULL, mguip_remember_layout_configuration, this, NULL),
    GNOMEUIINFO_END };

  GnomeUIInfo helpmenus[] = {
    //    GNOMEUIINFO_HELP("GMidnight"), 
    GNOMEUIINFO_MENU_ABOUT_ITEM(gguip_about, this),
    GNOMEUIINFO_END };

  GnomeUIInfo menus[] = { GNOMEUIINFO_SUBTREE("File", &filemenus),
			  GNOMEUIINFO_SUBTREE("Karaoke", &karamenus),
			  GNOMEUIINFO_SUBTREE("Configuration", &midiconfigmenus),
			  GNOMEUIINFO_MENU_HELP_TREE(&helpmenus),
			  GNOMEUIINFO_END };
  gnome_app_create_menus(GNOME_APP(main), menus);
  
  _showListPanelItem = filemenus[1].widget;
  _showKaraokePanelItem = karamenus[0].widget;
  
  _tbHasTextCheckMenuItem = midiconfigmenus[3].widget;
  
  gboolean def;
  gboolean b =  gnome_config_get_bool_with_default("gmidnight/layout/remember layout",&b);
  
  if (def) // the gnome default is to remember the layout of the docked things
    b = def;
  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(midiconfigmenus[4].widget),b);
}

void GnomeGUIPlayer::setPlayPixAsPlay()
{ 
  gnome_pixmap_load_xpm_d(GNOME_PIXMAP(_playPixmap), play_xpm);
}

void GnomeGUIPlayer::setPlayPixAsPause()
{
  gnome_pixmap_load_xpm_d(GNOME_PIXMAP(_playPixmap), pause_xpm);
}

void GnomeGUIPlayer::setToolbarHasText(gboolean b)
{
  gtk_toolbar_set_style(GTK_TOOLBAR(_commandToolbar), b?GTK_TOOLBAR_BOTH:GTK_TOOLBAR_ICONS);
}

void GnomeGUIPlayer::setRememberLayout(gboolean b)
{
  gnome_app_enable_layout_config  (GNOME_APP(_mainWindow), b);
  gnome_config_set_bool("gmidnight/layout/remember layout",b);
  gnome_config_sync();
}

void GnomeGUIPlayer::showKaraokePrefs()
{
  if (_karaoke != 0)
    {
      _karaoke->showPreferences();
    }
}
