/*
  Midnight, a C++ midi library to load and play midi files.
  Copyright (C) 1999 Rodrigo Reyes, reyes@chez.com

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef _MIDNIGHTGUIPLAYER_H_
#define _MIDNIGHTGUIPLAYER_H_

#include "GUIPlayer.h"
#include "MidiDevice.h"
#include "MidiFile.h"
#include "MidnightConfiguration.h"
#include "MidnightDevicePrefs.h"
#include "MidnightMidiPlayer.h"
#include "KaraokeBase.h"
#include "MidnightPlayerManager.h"
#include "MidnightList.h"
#include "PlayerInterface.h"
#include <iostream>
#include <fstream>

void mguip_select_device(GtkWidget* w, gpointer data);
void mguip_open(GtkWidget* w, gpointer data);
void mguip_save_config(GtkWidget* w, gpointer data);
void mguip_quit(GtkWidget* w, gpointer data);
void mguip_show_hide_panel(GtkWidget* w, gpointer data);
void mguip_karaoke_prefs(GtkWidget* w, gpointer data);
void mguip_show_hide_list(GtkWidget* w, gpointer data);

class MidnightGUIPlayer : public GUIPlayer // , public PlayerInterface
{
  MidnightPlayerManager*  _playermanager;
  MidnightList*           _titleList;

  GtkItemFactory*      _itemFactory;

 protected:
  KaraokeBase*         _karaoke;
  GtkWidget*  _showListPanelItem;
  GtkWidget*  _showKaraokePanelItem;

  vector<string> _titleText, _infoText;
  string language;

  class SelectDevicePrefs : public MidnightDevicePrefs
    {
      MidnightGUIPlayer* _guiplayer;
    public:
      SelectDevicePrefs(MidnightConfiguration*conf, MidnightGUIPlayer* player) : MidnightDevicePrefs(conf), _guiplayer(player)
	{
	}
      virtual void applyChanges()
	{
	  _guiplayer->pause();
	  //
	  //
	  // WAIT, this is a race condition to avoid a deadlock with
	  // an other thread accessing to the device while we destroy
	  // and change it.
	  //
	  _guiplayer->checkNotPlaying();

	  MidnightDevicePrefs::applyChanges();
	  _guiplayer->closedSelectDeviceWindow();
	}
      virtual void cancel()
	{
	  MidnightDevicePrefs::cancel();
	  _guiplayer->closedSelectDeviceWindow();
	}
    };

  SelectDevicePrefs* _selectDevice;
  
 public:

  MidnightGUIPlayer(MidnightPlayerManager* manager);
  virtual void init();
  virtual void attachMenuToMain(GtkWidget* main, GtkWidget* vbox);
  virtual ~MidnightGUIPlayer();

  virtual void previousSong();
  virtual void nextSong();
  virtual void selectSong(string& filename);
  virtual void itemFactoryAdd(GtkItemFactory *item_factory);
  void selectDevice();


  virtual void setFeature(string, string);
  virtual string getFeature(string, string);
  virtual void setIntFeature(string, int);
  virtual int getIntFeature(string, int);
  virtual void flushConfiguration() ;  

  int ticksToMs(int ticks)
    {
      //      bool b = _playermanager->isPlaying();
      return _playermanager->ticksToMs(ticks);
      //      return _playermanager->ticksToMs(ticks);
      //c      return 0;
    }

  void closedSelectDeviceWindow();
  void saveConfig();
  void setKaraokePanelVisibility(bool v);

  void toggleShowKaraokePanel();
  void toggleShowListPanel();

  virtual void showKaraokePrefs();  
  virtual void newFileSelected();

  bool _karaokeHidden;
  virtual void stop();
  virtual void play();

  virtual void setSongAt(unsigned long ticks);

  virtual void checkNotPlaying();

  void consumeData();
  virtual void addToList(vector<string> &lst);

  void toggleMenuItem_show_karaoke(bool val);
  void toggleMenuItem_show_list(bool val);
};

#endif
