/*
  Midnight, a C++ midi library to load and play midi files.
  Copyright (C) 1999 Rodrigo Reyes, reyes@chez.com

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "MidnightMidiPlayer.h"


MidnightMidiPlayer::MidnightMidiPlayer(MidiFile *file, MidiDevice *device) : MidiPlayer(file,device)
{
  _msIdle = 0;
  _msIdleSync = 0;
  _prevIdle = 0;
  _eventAny = true;
  _eventLyric = true;
  _eventText = true;
  _resetEventAny = true;
  //  _karaoke = NULL;
}


void MidnightMidiPlayer::setPosition(unsigned long ticks)
{
  /*      stop(); */
  MidiPlayer::setPosition(ticks);
      
  _msIdleSync = 0;
  _msIdle = 0;
  _prevIdle = 0;
  _resetEventAny = true;
  //      stop();
  //      play();
}

void MidnightMidiPlayer::stop()
{
  MidiPlayer::stop();
  _msIdle = 0;
  _prevIdle = 0;
}



void MidnightMidiPlayer::eventAny(vector<pair<unsigned long, MidiEvent*> >::iterator i)   
{
  if (_resetEventAny)
    {
      _msIdle = 0;
      _prevIdle = i->first;
      _resetEventAny = false;
    }

  //
  // Every 200ms, let's call gtk's main loop
  //
  _msIdle += (i->first - _prevIdle);
  _prevIdle = i->first;

  unsigned long ms = getDevice()->ticksToMs(getFile()->getDivision(), _msIdle);
  if (ms > 200)
    {
      _msIdle = 0;
      _msIdleSync += ms;

      if (_guiplayer != NULL)
	{
	  _guiplayer->setPosition(i->first, "    ", false);
	}

      while (gtk_events_pending())
	gtk_main_iteration();

    }
}

