/*
  Midnight, a C++ midi library to load and play midi files.
  Copyright (C) 1999 Rodrigo Reyes, reyes@chez.com

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef _MIDNIGHTMIDIPLAYER_H_
#define _MIDNIGHTMIDIPLAYER_H_

#include "MidiPlayer.h"
#include "GUIPlayer.h"
#include "MidiDevice.h"
#include "MidiFile.h"
#include "MidnightConfiguration.h"
#include "MidnightDevicePrefs.h"
//#include "KaraokeManager.h"

//
// Let's specialize the MidiPlayer class to make it 
// karaoke-aware and GUI-aware :
//
// - It sends position (time offset) information to the gui player is
// any (so that it can graphically show the current play time)
//
// - It sends Lyrics events to a KaraokeBase, if any.
//

class MidnightMidiPlayer : public MidiPlayer
{
  unsigned long _msIdle;
  unsigned long _msIdleSync;
  unsigned long _prevIdle;
  GUIPlayer* _guiplayer;
  bool _resetEventAny;
  //  KaraokeManager *_karaoke;

public:

  MidnightMidiPlayer(MidiFile *file, MidiDevice *device);

  void setGUIPlayer(GUIPlayer *guiplayer) { _guiplayer = guiplayer; }
  void setPosition(unsigned long ticks);
  virtual void eventAny(vector<pair<unsigned long, MidiEvent*> >::iterator i);
  virtual void stop();

  virtual void eventLyric(vector<pair<unsigned long, MidiEvent*> >::iterator i)   {  }
  virtual void eventText(vector<pair<unsigned long, MidiEvent*> >::iterator i)    {  }

  virtual void setFile(MidiFile *file)
    {
      MidiPlayer::setFile(file);
    }

};



#endif
